@@ -1,7 +1,15 @@
-"""Provide the 'autogenerate' feature which can produce migration operations
-automatically."""
+from __future__ import annotations
 
 import contextlib
+from typing import Any
+from typing import Callable
+from typing import Dict
+from typing import Iterator
+from typing import Optional
+from typing import Sequence
+from typing import Set
+from typing import TYPE_CHECKING
+from typing import Union
 
 from sqlalchemy import inspect
 
@@ -10,8 +18,29 @@ from . import render
 from .. import util
 from ..operations import ops
 
+"""Provide the 'autogenerate' feature which can produce migration operations
+automatically."""
 
-def compare_metadata(context, metadata):
+if TYPE_CHECKING:
+    from sqlalchemy.engine import Connection
+    from sqlalchemy.engine import Dialect
+    from sqlalchemy.engine import Inspector
+    from sqlalchemy.sql.schema import MetaData
+    from sqlalchemy.sql.schema import SchemaItem
+
+    from ..config import Config
+    from ..operations.ops import DowngradeOps
+    from ..operations.ops import MigrationScript
+    from ..operations.ops import UpgradeOps
+    from ..runtime.environment import NameFilterParentNames
+    from ..runtime.environment import NameFilterType
+    from ..runtime.environment import RenderItemFn
+    from ..runtime.migration import MigrationContext
+    from ..script.base import Script
+    from ..script.base import ScriptDirectory
+
+
+def compare_metadata(context: MigrationContext, metadata: MetaData) -> Any:
     """Compare a database schema to that given in a
     :class:`~sqlalchemy.schema.MetaData` instance.
 
@@ -28,36 +57,42 @@ def compare_metadata(context, metadata):
 
         from alembic.migration import MigrationContext
         from alembic.autogenerate import compare_metadata
-        from sqlalchemy.schema import SchemaItem
-        from sqlalchemy.types import TypeEngine
-        from sqlalchemy import (create_engine, MetaData, Column,
-                Integer, String, Table, text)
+        from sqlalchemy import (
+            create_engine,
+            MetaData,
+            Column,
+            Integer,
+            String,
+            Table,
+            text,
+        )
         import pprint
 
         engine = create_engine("sqlite://")
 
         with engine.begin() as conn:
-            conn.execute(text('''
-                create table foo (
-                    id integer not null primary key,
-                    old_data varchar,
-                    x integer
-                )'''))
-
-            conn.execute(text('''
-                create table bar (
-                    data varchar
-                )'''))
+            conn.execute(
+                text(
+                    '''
+                        create table foo (
+                            id integer not null primary key,
+                            old_data varchar,
+                            x integer
+                        )
+                    '''
+                )
+            )
+            conn.execute(text("create table bar (data varchar)"))
 
         metadata = MetaData()
-        Table('foo', metadata,
-            Column('id', Integer, primary_key=True),
-            Column('data', Integer),
-            Column('x', Integer, nullable=False)
-        )
-        Table('bat', metadata,
-            Column('info', String)
+        Table(
+            "foo",
+            metadata,
+            Column("id", Integer, primary_key=True),
+            Column("data", Integer),
+            Column("x", Integer, nullable=False),
         )
+        Table("bat", metadata, Column("info", String))
 
         mc = MigrationContext.configure(engine.connect())
 
@@ -66,29 +101,53 @@ def compare_metadata(context, metadata):
 
     Output::
 
-        [ ( 'add_table',
-            Table('bat', MetaData(bind=None),
-                Column('info', String(), table=<bat>), schema=None)),
-          ( 'remove_table',
-            Table(u'bar', MetaData(bind=None),
-                Column(u'data', VARCHAR(), table=<bar>), schema=None)),
-          ( 'add_column',
-            None,
-            'foo',
-            Column('data', Integer(), table=<foo>)),
-          ( 'remove_column',
-            None,
-            'foo',
-            Column(u'old_data', VARCHAR(), table=None)),
-          [ ( 'modify_nullable',
-              None,
-              'foo',
-              u'x',
-              { 'existing_server_default': None,
-                'existing_type': INTEGER()},
-              True,
-              False)]]
-
+        [
+            (
+                "add_table",
+                Table(
+                    "bat",
+                    MetaData(),
+                    Column("info", String(), table=<bat>),
+                    schema=None,
+                ),
+            ),
+            (
+                "remove_table",
+                Table(
+                    "bar",
+                    MetaData(),
+                    Column("data", VARCHAR(), table=<bar>),
+                    schema=None,
+                ),
+            ),
+            (
+                "add_column",
+                None,
+                "foo",
+                Column("data", Integer(), table=<foo>),
+            ),
+            [
+                (
+                    "modify_nullable",
+                    None,
+                    "foo",
+                    "x",
+                    {
+                        "existing_comment": None,
+                        "existing_server_default": False,
+                        "existing_type": INTEGER(),
+                    },
+                    True,
+                    False,
+                )
+            ],
+            (
+                "remove_column",
+                None,
+                "foo",
+                Column("old_data", VARCHAR(), table=<foo>),
+            ),
+        ]
 
     :param context: a :class:`.MigrationContext`
      instance.
@@ -106,7 +165,9 @@ def compare_metadata(context, metadata):
     return migration_script.upgrade_ops.as_diffs()
 
 
-def produce_migrations(context, metadata):
+def produce_migrations(
+    context: MigrationContext, metadata: MetaData
+) -> MigrationScript:
     """Produce a :class:`.MigrationScript` structure based on schema
     comparison.
 
@@ -136,26 +197,39 @@ def produce_migrations(context, metadata):
 
 
 def render_python_code(
-    up_or_down_op,
-    sqlalchemy_module_prefix="sa.",
-    alembic_module_prefix="op.",
-    render_as_batch=False,
-    imports=(),
-    render_item=None,
-    migration_context=None,
-):
+    up_or_down_op: Union[UpgradeOps, DowngradeOps],
+    sqlalchemy_module_prefix: str = "sa.",
+    alembic_module_prefix: str = "op.",
+    render_as_batch: bool = False,
+    imports: Sequence[str] = (),
+    render_item: Optional[RenderItemFn] = None,
+    migration_context: Optional[MigrationContext] = None,
+    user_module_prefix: Optional[str] = None,
+) -> str:
     """Render Python code given an :class:`.UpgradeOps` or
     :class:`.DowngradeOps` object.
 
     This is a convenience function that can be used to test the
     autogenerate output of a user-defined :class:`.MigrationScript` structure.
 
+    :param up_or_down_op: :class:`.UpgradeOps` or :class:`.DowngradeOps` object
+    :param sqlalchemy_module_prefix: module prefix for SQLAlchemy objects
+    :param alembic_module_prefix: module prefix for Alembic constructs
+    :param render_as_batch: use "batch operations" style for rendering
+    :param imports: sequence of import symbols to add
+    :param render_item: callable to render items
+    :param migration_context: optional :class:`.MigrationContext`
+    :param user_module_prefix: optional string prefix for user-defined types
+
+     .. versionadded:: 1.11.0
+
     """
     opts = {
         "sqlalchemy_module_prefix": sqlalchemy_module_prefix,
         "alembic_module_prefix": alembic_module_prefix,
         "render_item": render_item,
         "render_as_batch": render_as_batch,
+        "user_module_prefix": user_module_prefix,
     }
 
     if migration_context is None:
@@ -173,7 +247,9 @@ def render_python_code(
     )
 
 
-def _render_migration_diffs(context, template_args):
+def _render_migration_diffs(
+    context: MigrationContext, template_args: Dict[Any, Any]
+) -> None:
     """legacy, used by test_autogen_composition at the moment"""
 
     autogen_context = AutogenContext(context)
@@ -192,11 +268,11 @@ def _render_migration_diffs(context, template_args):
     )
 
 
-class AutogenContext(object):
+class AutogenContext:
     """Maintains configuration and state that's specific to an
     autogenerate operation."""
 
-    metadata = None
+    metadata: Optional[MetaData] = None
     """The :class:`~sqlalchemy.schema.MetaData` object
     representing the destination.
 
@@ -214,7 +290,7 @@ class AutogenContext(object):
 
     """
 
-    connection = None
+    connection: Optional[Connection] = None
     """The :class:`~sqlalchemy.engine.base.Connection` object currently
     connected to the database backend being compared.
 
@@ -223,7 +299,7 @@ class AutogenContext(object):
 
     """
 
-    dialect = None
+    dialect: Optional[Dialect] = None
     """The :class:`~sqlalchemy.engine.Dialect` object currently in use.
 
     This is normally obtained from the
@@ -231,7 +307,7 @@ class AutogenContext(object):
 
     """
 
-    imports = None
+    imports: Set[str] = None  # type: ignore[assignment]
     """A ``set()`` which contains string Python import directives.
 
     The directives are to be rendered into the ``${imports}`` section
@@ -245,13 +321,16 @@ class AutogenContext(object):
 
     """
 
-    migration_context = None
+    migration_context: MigrationContext = None  # type: ignore[assignment]
     """The :class:`.MigrationContext` established by the ``env.py`` script."""
 
     def __init__(
-        self, migration_context, metadata=None, opts=None, autogenerate=True
-    ):
-
+        self,
+        migration_context: MigrationContext,
+        metadata: Optional[MetaData] = None,
+        opts: Optional[dict] = None,
+        autogenerate: bool = True,
+    ) -> None:
         if (
             autogenerate
             and migration_context is not None
@@ -301,20 +380,30 @@ class AutogenContext(object):
             self.dialect = self.migration_context.dialect
 
         self.imports = set()
-        self.opts = opts
-        self._has_batch = False
+        self.opts: Dict[str, Any] = opts
+        self._has_batch: bool = False
 
     @util.memoized_property
-    def inspector(self):
+    def inspector(self) -> Inspector:
+        if self.connection is None:
+            raise TypeError(
+                "can't return inspector as this "
+                "AutogenContext has no database connection"
+            )
         return inspect(self.connection)
 
     @contextlib.contextmanager
-    def _within_batch(self):
+    def _within_batch(self) -> Iterator[None]:
         self._has_batch = True
         yield
         self._has_batch = False
 
-    def run_name_filters(self, name, type_, parent_names):
+    def run_name_filters(
+        self,
+        name: Optional[str],
+        type_: NameFilterType,
+        parent_names: NameFilterParentNames,
+    ) -> bool:
         """Run the context's name filters and return True if the targets
         should be part of the autogenerate operation.
 
@@ -325,29 +414,35 @@ class AutogenContext(object):
         :paramref:`.EnvironmentContext.configure.include_name` parameter.
 
         """
-
         if "schema_name" in parent_names:
             if type_ == "table":
                 table_name = name
             else:
-                table_name = parent_names["table_name"]
-            schema_name = parent_names["schema_name"]
-            if schema_name:
-                parent_names["schema_qualified_table_name"] = "%s.%s" % (
-                    schema_name,
-                    table_name,
-                )
-            else:
-                parent_names["schema_qualified_table_name"] = table_name
+                table_name = parent_names.get("table_name", None)
+            if table_name:
+                schema_name = parent_names["schema_name"]
+                if schema_name:
+                    parent_names["schema_qualified_table_name"] = "%s.%s" % (
+                        schema_name,
+                        table_name,
+                    )
+                else:
+                    parent_names["schema_qualified_table_name"] = table_name
 
         for fn in self._name_filters:
-
             if not fn(name, type_, parent_names):
                 return False
         else:
             return True
 
-    def run_object_filters(self, object_, name, type_, reflected, compare_to):
+    def run_object_filters(
+        self,
+        object_: SchemaItem,
+        name: Optional[str],
+        type_: NameFilterType,
+        reflected: bool,
+        compare_to: Optional[SchemaItem],
+    ) -> bool:
         """Run the context's object filters and return True if the targets
         should be part of the autogenerate operation.
 
@@ -368,7 +463,8 @@ class AutogenContext(object):
 
     @util.memoized_property
     def sorted_tables(self):
-        """Return an aggregate of the :attr:`.MetaData.sorted_tables` collection(s).
+        """Return an aggregate of the :attr:`.MetaData.sorted_tables`
+        collection(s).
 
         For a sequence of :class:`.MetaData` objects, this
         concatenates the :attr:`.MetaData.sorted_tables` collection
@@ -407,17 +503,17 @@ class AutogenContext(object):
         return result
 
 
-class RevisionContext(object):
+class RevisionContext:
     """Maintains configuration and state that's specific to a revision
     file generation operation."""
 
     def __init__(
         self,
-        config,
-        script_directory,
-        command_args,
-        process_revision_directives=None,
-    ):
+        config: Config,
+        script_directory: ScriptDirectory,
+        command_args: Dict[str, Any],
+        process_revision_directives: Optional[Callable] = None,
+    ) -> None:
         self.config = config
         self.script_directory = script_directory
         self.command_args = command_args
@@ -428,10 +524,10 @@ class RevisionContext(object):
         }
         self.generated_revisions = [self._default_revision()]
 
-    def _to_script(self, migration_script):
-        template_args = {}
-        for k, v in self.template_args.items():
-            template_args.setdefault(k, v)
+    def _to_script(
+        self, migration_script: MigrationScript
+    ) -> Optional[Script]:
+        template_args: Dict[str, Any] = self.template_args.copy()
 
         if getattr(migration_script, "_needs_render", False):
             autogen_context = self._last_autogen_context
@@ -445,6 +541,7 @@ class RevisionContext(object):
                 autogen_context, migration_script, template_args
             )
 
+        assert migration_script.rev_id is not None
         return self.script_directory.generate_revision(
             migration_script.rev_id,
             migration_script.message,
@@ -454,16 +551,25 @@ class RevisionContext(object):
             branch_labels=migration_script.branch_label,
             version_path=migration_script.version_path,
             depends_on=migration_script.depends_on,
-            **template_args
+            **template_args,
         )
 
-    def run_autogenerate(self, rev, migration_context):
+    def run_autogenerate(
+        self, rev: tuple, migration_context: MigrationContext
+    ) -> None:
         self._run_environment(rev, migration_context, True)
 
-    def run_no_autogenerate(self, rev, migration_context):
+    def run_no_autogenerate(
+        self, rev: tuple, migration_context: MigrationContext
+    ) -> None:
         self._run_environment(rev, migration_context, False)
 
-    def _run_environment(self, rev, migration_context, autogenerate):
+    def _run_environment(
+        self,
+        rev: tuple,
+        migration_context: MigrationContext,
+        autogenerate: bool,
+    ) -> None:
         if autogenerate:
             if self.command_args["sql"]:
                 raise util.CommandError(
@@ -492,9 +598,10 @@ class RevisionContext(object):
                 ops.DowngradeOps([], downgrade_token=downgrade_token)
             )
 
-        self._last_autogen_context = autogen_context = AutogenContext(
+        autogen_context = AutogenContext(
             migration_context, autogenerate=autogenerate
         )
+        self._last_autogen_context: AutogenContext = autogen_context
 
         if autogenerate:
             compare._populate_migration_script(
@@ -513,20 +620,21 @@ class RevisionContext(object):
         for migration_script in self.generated_revisions:
             migration_script._needs_render = True
 
-    def _default_revision(self):
+    def _default_revision(self) -> MigrationScript:
+        command_args: Dict[str, Any] = self.command_args
         op = ops.MigrationScript(
-            rev_id=self.command_args["rev_id"] or util.rev_id(),
-            message=self.command_args["message"],
+            rev_id=command_args["rev_id"] or util.rev_id(),
+            message=command_args["message"],
             upgrade_ops=ops.UpgradeOps([]),
             downgrade_ops=ops.DowngradeOps([]),
-            head=self.command_args["head"],
-            splice=self.command_args["splice"],
-            branch_label=self.command_args["branch_label"],
-            version_path=self.command_args["version_path"],
-            depends_on=self.command_args["depends_on"],
+            head=command_args["head"],
+            splice=command_args["splice"],
+            branch_label=command_args["branch_label"],
+            version_path=command_args["version_path"],
+            depends_on=command_args["depends_on"],
         )
         return op
 
-    def generate_scripts(self):
+    def generate_scripts(self) -> Iterator[Optional[Script]]:
         for generated_revision in self.generated_revisions:
             yield self._to_script(generated_revision)
