@@ -26,6 +26,7 @@ from ..util import sqla_compat
 if TYPE_CHECKING:
     from typing import Literal
 
+    from sqlalchemy.sql.base import DialectKWArgs
     from sqlalchemy.sql.elements import ColumnElement
     from sqlalchemy.sql.elements import TextClause
     from sqlalchemy.sql.schema import CheckConstraint
@@ -94,7 +95,6 @@ def _render_cmd_body(
     op_container: ops.OpContainer,
     autogen_context: AutogenContext,
 ) -> str:
-
     buf = StringIO()
     printer = PythonPrinter(buf)
 
@@ -165,7 +165,6 @@ def _render_modify_table(
 def _render_create_table_comment(
     autogen_context: AutogenContext, op: ops.CreateTableCommentOp
 ) -> str:
-
     templ = (
         "{prefix}create_table_comment(\n"
         "{indent}'{tname}',\n"
@@ -190,7 +189,6 @@ def _render_create_table_comment(
 def _render_drop_table_comment(
     autogen_context: AutogenContext, op: ops.DropTableCommentOp
 ) -> str:
-
     templ = (
         "{prefix}drop_table_comment(\n"
         "{indent}'{tname}',\n"
@@ -271,6 +269,15 @@ def _drop_table(autogen_context: AutogenContext, op: ops.DropTableOp) -> str:
     return text
 
 
+def _render_dialect_kwargs_items(
+    autogen_context: AutogenContext, item: DialectKWArgs
+) -> list[str]:
+    return [
+        f"{key}={_render_potential_expr(val, autogen_context)}"
+        for key, val in item.dialect_kwargs.items()
+    ]
+
+
 @renderers.dispatch_for(ops.CreateIndexOp)
 def _add_index(autogen_context: AutogenContext, op: ops.CreateIndexOp) -> str:
     index = op.to_index()
@@ -289,6 +296,8 @@ def _add_index(autogen_context: AutogenContext, op: ops.CreateIndexOp) -> str:
         )
 
     assert index.table is not None
+
+    opts = _render_dialect_kwargs_items(autogen_context, index)
     text = tmpl % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "name": _render_gen_name(autogen_context, index.name),
@@ -300,18 +309,7 @@ def _add_index(autogen_context: AutogenContext, op: ops.CreateIndexOp) -> str:
         "schema": (", schema=%r" % _ident(index.table.schema))
         if index.table.schema
         else "",
-        "kwargs": (
-            ", "
-            + ", ".join(
-                [
-                    "%s=%s"
-                    % (key, _render_potential_expr(val, autogen_context))
-                    for key, val in index.kwargs.items()
-                ]
-            )
-        )
-        if len(index.kwargs)
-        else "",
+        "kwargs": ", " + ", ".join(opts) if opts else "",
     }
     return text
 
@@ -329,24 +327,13 @@ def _drop_index(autogen_context: AutogenContext, op: ops.DropIndexOp) -> str:
             "%(prefix)sdrop_index(%(name)r, "
             "table_name=%(table_name)r%(schema)s%(kwargs)s)"
         )
-
+    opts = _render_dialect_kwargs_items(autogen_context, index)
     text = tmpl % {
         "prefix": _alembic_autogenerate_prefix(autogen_context),
         "name": _render_gen_name(autogen_context, op.index_name),
         "table_name": _ident(op.table_name),
         "schema": ((", schema=%r" % _ident(op.schema)) if op.schema else ""),
-        "kwargs": (
-            ", "
-            + ", ".join(
-                [
-                    "%s=%s"
-                    % (key, _render_potential_expr(val, autogen_context))
-                    for key, val in index.kwargs.items()
-                ]
-            )
-        )
-        if len(index.kwargs)
-        else "",
+        "kwargs": ", " + ", ".join(opts) if opts else "",
     }
     return text
 
@@ -362,7 +349,6 @@ def _add_unique_constraint(
 def _add_fk_constraint(
     autogen_context: AutogenContext, op: ops.CreateForeignKeyOp
 ) -> str:
-
     args = [repr(_render_gen_name(autogen_context, op.constraint_name))]
     if not autogen_context._has_batch:
         args.append(repr(_ident(op.source_table)))
@@ -411,7 +397,6 @@ def _add_check_constraint(constraint, autogen_context):
 def _drop_constraint(
     autogen_context: AutogenContext, op: ops.DropConstraintOp
 ) -> str:
-
     if autogen_context._has_batch:
         template = "%(prefix)sdrop_constraint" "(%(name)r, type_=%(type)r)"
     else:
@@ -432,7 +417,6 @@ def _drop_constraint(
 
 @renderers.dispatch_for(ops.AddColumnOp)
 def _add_column(autogen_context: AutogenContext, op: ops.AddColumnOp) -> str:
-
     schema, tname, column = op.schema, op.table_name, op.column
     if autogen_context._has_batch:
         template = "%(prefix)sadd_column(%(column)s)"
@@ -452,7 +436,6 @@ def _add_column(autogen_context: AutogenContext, op: ops.AddColumnOp) -> str:
 
 @renderers.dispatch_for(ops.DropColumnOp)
 def _drop_column(autogen_context: AutogenContext, op: ops.DropColumnOp) -> str:
-
     schema, tname, column_name = op.schema, op.table_name, op.column_name
 
     if autogen_context._has_batch:
@@ -476,7 +459,6 @@ def _drop_column(autogen_context: AutogenContext, op: ops.DropColumnOp) -> str:
 def _alter_column(
     autogen_context: AutogenContext, op: ops.AlterColumnOp
 ) -> str:
-
     tname = op.table_name
     cname = op.column_name
     server_default = op.modify_server_default
@@ -566,7 +548,6 @@ def _render_potential_expr(
     is_server_default: bool = False,
 ) -> str:
     if isinstance(value, sql.ClauseElement):
-
         if wrap_in_text:
             template = "%(prefix)stext(%(sql)r)"
         else:
@@ -613,6 +594,7 @@ def _uq_constraint(
         opts.append(
             ("name", _render_gen_name(autogen_context, constraint.name))
         )
+    dialect_options = _render_dialect_kwargs_items(autogen_context, constraint)
 
     if alter:
         args = [repr(_render_gen_name(autogen_context, constraint.name))]
@@ -620,6 +602,7 @@ def _uq_constraint(
             args += [repr(_ident(constraint.table.name))]
         args.append(repr([_ident(col.name) for col in constraint.columns]))
         args.extend(["%s=%r" % (k, v) for k, v in opts])
+        args.extend(dialect_options)
         return "%(prefix)screate_unique_constraint(%(args)s)" % {
             "prefix": _alembic_autogenerate_prefix(autogen_context),
             "args": ", ".join(args),
@@ -627,6 +610,7 @@ def _uq_constraint(
     else:
         args = [repr(_ident(col.name)) for col in constraint.columns]
         args.extend(["%s=%r" % (k, v) for k, v in opts])
+        args.extend(dialect_options)
         return "%(prefix)sUniqueConstraint(%(args)s)" % {
             "prefix": _sqlalchemy_autogenerate_prefix(autogen_context),
             "args": ", ".join(args),
@@ -675,7 +659,6 @@ def _render_column(
     opts: List[Tuple[str, Any]] = []
 
     if column.server_default:
-
         rendered = _render_server_default(  # type:ignore[assignment]
             column.server_default, autogen_context
         )
@@ -984,7 +967,6 @@ def _fk_colspec(
 def _populate_render_fk_opts(
     constraint: ForeignKeyConstraint, opts: List[Tuple[str, str]]
 ) -> None:
-
     if constraint.onupdate:
         opts.append(("onupdate", repr(constraint.onupdate)))
     if constraint.ondelete:
