@@ -571,8 +571,6 @@ class DefaultImpl(metaclass=ImplMeta):
         """Render a SQL expression that is typically a server default,
         index expression, etc.
 
-        .. versionadded:: 1.0.11
-
         """
 
         compile_kw = {
@@ -691,6 +689,11 @@ class DefaultImpl(metaclass=ImplMeta):
                 )
                 metadata_indexes.discard(idx)
 
+    def adjust_reflected_dialect_options(
+        self, reflected_object: Dict[str, Any], kind: str
+    ) -> Dict[str, Any]:
+        return reflected_object.get("dialect_options", {})
+
 
 def _compare_identity_options(
     attributes, metadata_io, inspector_io, default_io
